@@ -55,7 +55,7 @@
 #define LNDMRK 74
 #define LIGHTS 75
 #define LOKBSN 79
-#define MARKUL 82
+#define MARCUL 82
 #define MORFAC 84
 #define NAVLNE 85
 #define OBSTRN 86
@@ -122,6 +122,7 @@
 #define I_GATCON 17031
 #define I_SLCONS 17032
 #define I_WTWAXS 17051
+#define I_BUNSTA 17054
 #define I_TERMNL 17064
 #define I_TRNBSN 17065
 #define I_WTWGAG 17067
